# The cutoff ratio defines the ratio of the gap that is to be ignored.
cutoffRatio = 0.6
# How many frequencies will be looked at after the similar frequencies count
similarFrequenciesCount = 10
# How stark the deviation has to be to count as different within the last "similarFrequenciesCount" frequencies
similarityStdDev = 0.05
dieGracefully = False

maxiLength = None
minimumIntensity = None